@@ -13,7 +13,7 @@ require Exporter;
 require HTTP::Request;
 use Carp();
 
-$VERSION = "5.817";
+$VERSION = "5.822";
 
 my $CRLF = "\015\012";   # "\r\n" is not portable
 
@@ -102,15 +102,19 @@ sub _simple_req
     my($method, $url) = splice(@_, 0, 2);
     my $req = HTTP::Request->new($method => $url);
     my($k, $v);
+    my $content;
     while (($k,$v) = splice(@_, 0, 2)) {
 	if (lc($k) eq 'content') {
 	    $req->add_content($v);
-            $req->header("Content-Length", length(${$req->content_ref}));
+            $content++;
 	}
 	else {
 	    $req->push_header($k, $v);
 	}
     }
+    if ($content && !defined($req->header("Content-Length"))) {
+        $req->header("Content-Length", length(${$req->content_ref}));
+    }
     $req;
 }
 
