@@ -1,8 +1,6 @@
 package WWW::RobotRules;
 
-# $Id: RobotRules.pm,v 1.29 2004/04/06 11:37:32 gisle Exp $
-
-$VERSION = sprintf("%d.%02d", q$Revision: 1.29 $ =~ /(\d+)\.(\d+)/);
+$VERSION = "5.822";
 sub Version { $VERSION; }
 
 use strict;
@@ -13,7 +11,7 @@ use URI ();
 sub new {
     my($class, $ua) = @_;
 
-    # This ugly hack is needed to ensure backwards compatability.
+    # This ugly hack is needed to ensure backwards compatibility.
     # The "WWW::RobotRules" class is now really abstract.
     $class = "WWW::RobotRules::InCore" if $class eq "WWW::RobotRules";
 
@@ -34,6 +32,7 @@ sub parse {
     my $ua;
     my $is_me = 0;		# 1 iff this record is for me
     my $is_anon = 0;		# 1 iff this record is for *
+    my $seen_disallow = 0;      # watch for missing record separators
     my @me_disallowed = ();	# rules disallowed for me
     my @anon_disallowed = ();	# rules disallowed for *
 
@@ -53,10 +52,19 @@ sub parse {
 	if (/^\s*$/) {	    # blank line
 	    last if $is_me; # That was our record. No need to read the rest.
 	    $is_anon = 0;
+	    $seen_disallow = 0;
 	}
         elsif (/^\s*User-Agent\s*:\s*(.*)/i) {
 	    $ua = $1;
 	    $ua =~ s/\s+$//;
+
+	    if ($seen_disallow) {
+		# treat as start of a new record
+		$seen_disallow = 0;
+		last if $is_me; # That was our record. No need to read the rest.
+		$is_anon = 0;
+	    }
+
 	    if ($is_me) {
 		# This record already had a User-agent that
 		# we matched, so just continue.
@@ -70,11 +78,12 @@ sub parse {
 	}
 	elsif (/^\s*Disallow\s*:\s*(.*)/i) {
 	    unless (defined $ua) {
-		warn "RobotRules: Disallow without preceding User-agent\n";
+		warn "RobotRules <$robot_txt_uri>: Disallow without preceding User-agent\n" if $^W;
 		$is_anon = 1;  # assume that User-agent: * was intended
 	    }
 	    my $disallow = $1;
 	    $disallow =~ s/\s+$//;
+	    $seen_disallow = 1;
 	    if (length $disallow) {
 		my $ignore;
 		eval {
@@ -97,7 +106,7 @@ sub parse {
 	    }
 	}
 	else {
-	    warn "RobotRules: Unexpected line: $_\n";
+	    warn "RobotRules <$robot_txt_uri>: Unexpected line: $_\n" if $^W;
 	}
     }
 
@@ -121,14 +130,10 @@ sub is_me {
     # See whether my short-name is a substring of the
     #  "User-Agent: ..." line that we were passed:
     
-    if(index(lc($ua_line), lc($me)) >= 0) {
-      LWP::Debug::debug("\"$ua_line\" applies to \"$me\"")
-       if defined &LWP::Debug::debug;
+    if(index(lc($me), lc($ua_line)) >= 0) {
       return 1;
     }
     else {
-      LWP::Debug::debug("\"$ua_line\" does not apply to \"$me\"")
-       if defined &LWP::Debug::debug;
       return '';
     }
 }
@@ -185,10 +190,12 @@ sub agent {
         #       "FooBot/1.2"                                  => "FooBot"
         #       "FooBot/1.2 [http://foobot.int; foo@bot.int]" => "FooBot"
 
-	delete $self->{'loc'};   # all old info is now stale
 	$name = $1 if $name =~ m/(\S+)/; # get first word
 	$name =~ s!/.*!!;  # get rid of version
-	$self->{'ua'}=$name;
+	unless ($old && $old eq $name) {
+	    delete $self->{'loc'}; # all old info is now stale
+	    $self->{'ua'} = $name;
+	}
     }
     $old;
 }
@@ -370,6 +377,14 @@ one robot. At least one field needs to be present per record.  If the
 value is '*', the record describes the default access policy for any
 robot that has not not matched any of the other records.
 
+The I<User-Agent> fields must occur before the I<Disallow> fields.  If a
+record contains a I<User-Agent> field after a I<Disallow> field, that
+constitutes a malformed record.  This parser will assume that a blank
+line should have been placed before that I<User-Agent> field, and will
+break the record into two.  All the fields before the I<User-Agent> field
+will constitute a record, and the I<User-Agent> field will be the first
+field in a new record.
+
 =item Disallow
 
 The value of this field specifies a partial URL that is not to be
@@ -404,6 +419,22 @@ This example indicates that no robots should visit this site further:
   User-agent: *
   Disallow: /
 
+This is an example of a malformed robots.txt file.
+
+  # robots.txt for ancientcastle.example.com
+  # I've locked myself away.
+  User-agent: *
+  Disallow: /
+  # The castle is your home now, so you can go anywhere you like.
+  User-agent: Belle
+  Disallow: /west-wing/ # except the west wing!
+  # It's good to be the Prince...
+  User-agent: Beast
+  Disallow: 
+
+This file is missing the required blank lines between records.
+However, the intention is clear.
+
 =head1 SEE ALSO
 
 L<LWP::RobotUA>, L<WWW::RobotRules::AnyDBM_File>
